# Migration Guide for `gatsby-source-contentful` v9

The v9 release of `gatsby-source-contentful` brings significant improvements, focusing on stability and enhancing the developer experience.

<details>
<summary><strong>Table of contents</strong></summary>

- [Migration Guide for `gatsby-source-contentful` v9](#migration-guide-for-gatsby-source-contentful-v9)
  - [Core Changes, Updates and new Features](#core-changes-updates-and-new-features)
  - [1. Introduction](#1-introduction)
  - [2. Automated Migration with Codemods](#2-automated-migration-with-codemods)
  - [3. Manual Changes and Additional Updates](#3-manual-changes-and-additional-updates)
  - [Breaking Changes](#breaking-changes)
    - [1. Remove Your Workarounds](#1-remove-your-workarounds)
    - [2. New content type naming pattern](#2-new-content-type-naming-pattern)
    - [3. Metadata / Data from Contentful's `sys` field](#3-metadata--data-from-contentfuls-sys-field)
      - [Sorting by `sys`](#sorting-by-sys)
      - [Filtering by `sys`](#filtering-by-sys)
    - [4. Fields](#4-fields)
      - [Text fields](#text-fields)
      - [JSON fields](#json-fields)
      - [Rich Text field](#rich-text-field)
        - [Schema Comparison](#schema-comparison)
        - [Query Updates](#query-updates)
        - [Rendering changes for Rich Text](#rendering-changes-for-rich-text)
    - [5. Assets](#5-assets)
      - [Old GraphlQL schema for assets](#old-graphlql-schema-for-assets)
      - [New GraphlQL schema for assets](#new-graphlql-schema-for-assets)
  - [Conclusion and Support](#conclusion-and-support)

</details>

## Core Changes, Updates and new Features

- **Dynamic Schema Generation**: Schema types are now dynamically generated based on the Contentful content model, eliminating site breakages due to empty content fields.
- **Rich Text and JSON Field Enhancements**: Improved handling of Rich Text and JSON fields for more accurate processing, querying, and rendering.
- **Schema and Performance Optimizations**: An optimized, leaner schema reduces node count and improves build times, especially for larger projects.
- **Contentful GraphQL API Alignment**: Enhanced alignment with Contentful's GraphQL API enables more intuitive and consistent querying.
- **Removed Content Type Name Restrictions**: New naming patterns now allow all content type names, including previously restricted names like `entity` and `reference`.
- **Updated Field Name Restrictions**: Fields can now be named `contentful_id`, with restrictions now applied to names like `sys`, `contentfulMetadata`, and `linkedFrom`.
- **Refined Backlinks**: Backlinks/references are now located in the `linkedFrom` field, aligning with Contentful's GraphQL API structure.
- **Expanded Configuration Options**: Additional [configuration options](#advanced-configuration) provide greater control and customization for your specific project needs.

## 1. Introduction

Version 9 of the `gatsby-source-contentful` plugin marks a significant evolution, introducing changes mainly in the structure of GraphQL queries.

This guide helps you to navigate through these updates, ensuring a straightforward migration to the latest version. We'll cover both automated and manual steps to adapt your project seamlessly to v9's advancements.

## 2. Automated Migration with Codemods

Before you start manual updates, take advantage of our codemods that automate most of the transition process. You can use one of the following options to apply the codemods to your project:

**Option A:** (Direct Execution)

```bash
npx gatsby-codemods@ctf-next gatsby-source-contentful --extensions=js,ts,tsx gatsby-* src
```

**Option B:** (Install, Execute, Uninstall)

```bash
# Install codemods
npm install -D jscodeshift gatsby-codemods@ctf-next

# Execute codemods
npx jscodeshift -t ./node_modules/gatsby-codemods/transforms/gatsby-source-contentful.js --extensions=js,ts,tsx gatsby-* src

# Uninstall codemods
npm remove jscodeshift gatsby-codemods
```

**Handling Large Codebases:** If your project is particularly large, you may need to increase the maximum memory allocation for the process:

```bash
NODE_OPTIONS=--max_old_space_size=10240 npx ...
```

## 3. Manual Changes and Additional Updates

After applying the codemods, review your project for any additional changes that need to be made manually. The following sections detail specific areas that require attention.

## Breaking Changes

With v9, the schema generated by `gatsby-source-contentful` is now closely aligned with Contentful's GraphQL API. This significant alignment means several changes in how you structure your GraphQL queries:

### 1. Remove Your Workarounds

If you previously implemented workarounds, such as freezing your schema with `gatsby-plugin-schema-snapshot` or using `createSchemaCustomization` to address issues in your Contentful GraphQL schema, you can safely remove these.

The enhancements in v9 have been designed to eliminate the need for such workarounds, offering a more reliable schema generation process that mirrors the structure and functionality of Contentful's native GraphQL API.

### 2. New content type naming pattern

The old naming pattern for generated node types based on your Contentful content model was pretty simple. This gave us shorter queries, but came with restrictions for content type names. Content type names like `entity`, `reference`, `tag`, `asset` did cause problems or simply did not work.

1. Generated GraphQL node types are now prefixed with `ContentfulContentType` instead of `Contentful`
2. Hardcoded types and interfaces stay short. For example: `ContentfulEntry`, `ContentfulAsset`, `ContentfulLocation` or
   `ContentfulRichText`

This is close to the [behaviour of the Contentful GraphQL API](https://www.contentful.com/developers/docs/references/graphql/#/reference/schema-generation/types):

> If the generated name starts with a number or collides with a reserved type name, it gets prefixed with 'ContentType'

Instead of doing this on a collision, we always prefix for consistency.

```diff
export const pageQuery = graphql`
  query {
-    allContentfulProduct {
+    allContentfulContentTypeProduct {
       ...
    }
  }
`
```

### 3. Metadata / Data from Contentful's `sys` field

The `sys` field now reflects the structure of the sys field in the Contentful [GraphQL API](https://www.contentful.com/developers/docs/references/graphql/#/reference/schema-generation/sys-field)

| Old Path        | New Path             | Comment                                                                                                                                                                                                        |
| --------------- | -------------------- | -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- |
| node_locale     | sys.locale           | this does not exists in Contentful's sys but we need it (for now) [as Contentful GraphQL handles locales different](https://www.contentful.com/developers/docs/references/graphql/#/reference/locale-handling) |
| contentful_id   | sys.id               |
| spaceId         | sys.spaceId          |
| createdAt       | sys.firstPublishedAt |
| updatedAt       | sys.publishedAt      |
| revision        | sys.publishedVersion |
|                 | sys.environmentId    | This is **new**                                                                                                                                                                                                |
| sys.contentType | sys.contentType      | This is now a real reference to the Content Type node. If you used this field, you will probably need to change the`__typename` subfield to `name`                                                             |

#### Sorting by `sys`

```graphql
allContentfulPage(sort: { fields: contentful_id }) { ... }
```

becomes

```graphql
allContentfulContentTypePage(sort: { fields: sys___id }) { ... }
```

#### Filtering by `sys`

```graphql
contentfulPage(
  contentful_id: {
    eq: "38akBjGb3T1t4AjB87wQjo"
  }
) { ... }
```

becomes

```graphql
contentfulContentTypePage(
  sys: {
    id: {
      eq: "38akBjGb3T1t4AjB87wQjo"
    }
  }
) { ... }
```

### 4. Fields

#### Text fields

- The raw field value is now stored in `FIELD_NAME { raw }` instead of `FIELD_NAME { FIELD_NAME }`
- This helps for consistency on your side, while reducing node type bload in the Gatsby backend

#### JSON fields

1. You can no more queries sub-fields, which ensures your queries won't break on changed content. This means you now have to check if the results have a value on your own.
   - You can achieve this with a [custom Field Editor](https://www.contentful.com/blog/apps-and-open-source-guide-to-making-contentful-truly-yours/) in Contentful.
2. Spaces within keys of JSON fields are now kept, means your data is accessible in you code the same way it is entered in contentful.

```jsx
<p data-cy-value-born-at>Born at: {actor.Born_At}</p>
```

becomes

```jsx
<p data-cy-value-born-at>Born at: {actor["Born At"]}</p>
```

#### Rich Text field

The schema for Rich Text fields in `gatsby-source-contentful` v9 has been updated to align with Contentful's GraphQL API. This change enhances the flexibility and consistency of querying Rich Text content. You will need to adjust your GraphQL queries and rendering code accordingly.

Key Changes:

1. **Unified Link Subfield**: You can now query all linked Contentful content types within the `links` subfield of Rich Text, irrespective of whether they are directly linked in the current content or not.
2. **Direct Entry ID Querying**: Instead of navigating through various fields, the entry ID can now be queried directly as a child field of `links`. This simplification makes your queries more concise.

##### Schema Comparison

**Old Schema:**

```graphql
type ContentfulRichTextRichText {
  raw: String
  references: [ContentfulAssetContentfulContentReferenceContentfulLocationContentfulTextUnion]
    @link(by: "id", from: "references___NODE")
}
union ContentfulAssetContentfulContentReferenceContentfulLocationContentfulTextUnion =
    ContentfulAsset
  | ContentfulContentReference
  | ContentfulLocation
  | ContentfulText
```

**New Schema:**

```graphql
type ContentfulNodeTypeRichText @dontInfer {
  json: JSON
  links: ContentfulNodeTypeRichTextLinks
}

type ContentfulNodeTypeRichTextLinks {
  assets: ContentfulNodeTypeRichTextAssets
  entries: ContentfulNodeTypeRichTextEntries
}

type ContentfulNodeTypeRichTextAssets {
  block: [ContentfulAsset]!
  hyperlink: [ContentfulAsset]!
}

type ContentfulNodeTypeRichTextEntries {
  inline: [ContentfulEntry]!
  block: [ContentfulEntry]!
  hyperlink: [ContentfulEntry]!
}
```

##### Query Updates

Update your Rich Text queries to reflect the new schema:

**Old Example Query:**

```graphql
query pageQuery {
  allContentfulRichText {
    nodes {
      richText {
        raw
        references {
          ... on ContentfulAsset {
            url
          }
          ... on ContentfulText {
            id
          }
        }
      }
    }
  }
}
```

**New Example Query:**

```graphql
query pageQuery {
  allContentfulContentTypeRichText {
    nodes {
      richText {
        json
        links {
          assets {
            block {
              url
            }
          }
          entries {
            block {
              ... on ContentfulContentTypeText {
                id
              }
            }
          }
        }
      }
    }
  }
}
```

**Notes**:

- The `raw` field is now `json`.
- The `references` field is replaced by `links`, providing a more structured and granular approach to handling linked content.

##### Rendering changes for Rich Text

Instead of passing your option object into `renderRichText()` you now pass a option factory. This allows you to access the data you queried for all the linked entities (`assetBlockMap`, `entryBlockMap`, `entryInlineMap`).

**Old rendering logic:**

```tsx
import { renderRichText } from "gatsby-source-contentful/rich-text"
import { Options } from "@contentful/rich-text-react-renderer"
import { BLOCKS, MARKS } from "@contentful/rich-text-types"

const options: Options = {
  renderNode: {
    [BLOCKS.EMBEDDED_ASSET]: node => {
      const image = getImage(node.data.target)
      return image ? (
        <GatsbyImage image={image} alt={node.data.target.description} />
      ) : null
    },
  },
}

;<div>{renderRichText(richText, options)}</div>
```

**New rendering logic:**

```tsx
import { renderRichText, MakeOptions } from "gatsby-source-contentful"
import { BLOCKS, MARKS } from "@contentful/rich-text-types"

const makeOptions: MakeOptions = ({
  assetBlockMap,
  assetHyperlinkMap,
  entryBlockMap,
  entryInlineMap,
  entryHyperlinkMap,
}) => ({
  renderNode: {
    [BLOCKS.EMBEDDED_ASSET]: node => {
      const image = assetBlockMap.get(node.data.target.sys.id)
      return image && image.gatsbyImageData ? (
        <GatsbyImage
          image={image.gatsbyImageData}
          alt={image.description || ""}
        />
      ) : null
    },
  },
})

;<div>{renderRichText(richText, makeOptions)}</div>
```

Find more details abour [rendering Rich Text in our README file section](./README.md#leveraging-contentful-rich-text-with-gatsby).

### 5. Assets

Assets got simplified a lot and aligned to the [Contentful GraphQL API](https://www.contentful.com/developers/docs/references/graphql/#/reference/schema-generation/assets).

#### Old GraphlQL schema for assets

```graphql
type ContentfulAsset implements ContentfulInternalReference & Node @dontInfer {
  file: ContentfulAssetFile
  title: String
  description: String
  sys: ContentfulInternalSys
}

type ContentfulAssetFile {
  url: String
  details: ContentfulAssetFileDetails
  fileName: String
  contentType: String
}

type ContentfulAssetFileDetails {
  size: Int
  image: ContentfulAssetFileDetailsImage
}

type ContentfulAssetFileDetailsImage {
  width: Int
  height: Int
}
```

#### New GraphlQL schema for assets

```graphql
type ContentfulAsset implements ContentfulInternalReference & Node @dontInfer {
  sys: ContentfulInternalSys
  title: String
  description: String
  mimeType: String
  fileName: String
  url: String
  size: Int
  width: Int
  height: Int
}
```

## Conclusion and Support

We understand that the changes introduced in `gatsby-source-contentful` v9 are significant. These updates were necessary to resolve architectural issues from the early stages of the plugin and to align it more closely with the evolving capabilities of Contentful and Gatsby.

As we step into this new era, our focus is on ensuring that you experience the most stable, efficient, and future-proof integration when using Contentful with Gatsby. While we've made every effort to simplify the migration process, we also recognize that transitions like these can be challenging.

If you encounter any difficulties or have questions, we encourage you to reach out for support:

- For detailed discussions and sharing experiences: [Join our GitHub Discussion](https://github.com/gatsbyjs/gatsby/discussions/38585).
- To report bugs or specific issues: [Submit an Issue on GitHub](https://github.com/gatsbyjs/gatsby/issues).
